#!/usr/bin/php
<?php

require_once '/usr/local/opmon/etc/config.php';
require_once '/usr/local/opmon/libexec/php-nagios-plugins/Nagios_Plugin.php';

define("VERSION", "2.1");

$np = null;

/*******************************************************************************
* Exibir o manual do software
********************************************************************************/
function getGuideLine()
{
    $name = basename(__FILE__);
    $realpath = realpath(__FILE__);
    print <<<EOF

------------------------------------------
Manual de exemplo.
------------------------------------------

Descrição do plugin:
    Executa o backup de switchs e armazena um arquivo com os dados no
    OpMon utilizando OpSCM (OpMon Source Configuration Manager).

Instalação:
    Copiar este plugin para a pasta de plugins no path abaixo:
    /usr/local/opmon/libexec/opservices.

    Dar permissão de execução( chmod +x $realpath )

Dependências:
    Nagios_Plugins para php

Funcionamento:
    Obtem as configurações de switchs executando comandos através de um shell interativo.

    Após o obter a configuração o plugin irá armazenar os dados no OpMon
    para possibilitar o controle de versão do mesmo.

Exemplos de execução:
    ./$name -H <nome do host no opmon> -i <credential id> -m <switch model> -v
    [modo verbose]

    ./$name -H <nome do host no opmon> -i <credential id> -m <switch model>
    [Obtem as configurações do switch (default)]

    ./$name -H <nome do host no opmon> -i <credential id> -m <switch model> -l "/#/:terminal length 0,/#/:show version,/#/"
    [Obtem as configurações do switch modo multi comando]


EOF;

    exit(OK);

}

/*******************************************************************************
* Seta os atributos da variável global $np
********************************************************************************/
function setNagios()
{

    # Constructor
    global $np;

    $np = new Nagios_Plugin(
        array (
            'version' => VERSION . " [www.opservices.com.br/suporte]",
            'blurb' => "Developed by:\n\tOpServices\n"
            . "Author: \n\tSidney Souza - sidney.souza@opservices.com.br\n",
            'usage' => "Usage:\n\t%s -H <host name> -i <credential-id> -m <model>\n"
            . " \t [-p <port>] [-I] [-v] [--man] [-h] [-?] [-c <command>]",
        )
    );

    $np->add_arg("protocol|P=s", "Protocol to connect to host");
    $np->add_arg("hostname|H=s", "Name of host in OpMon");
    $np->add_arg("port|p=i", "Port to connect");
    $np->add_arg("credential-id|i=i", "Credential id");
    $np->add_arg("m|model=s", "Switch Manufacturer)");
    $np->add_arg("man", "Show guideline");
    $np->add_arg("run", "Run backup in background");
    $np->add_arg("c|backup-command=s", "Custom command that will be used to perform backup");
    $np->add_arg("s|service-description=s", "Custom service_description");
    $np->add_arg("t|timeout=i", "The time from each command will wait to stop");
    $np->add_arg(
        "l|command-list=s",
        "Commands list that need to execute to do a backup"
        . " ex: <expected_regex>:<command>,<expected_regex>:<command>"
        . "\n\n*ATTENTION*: You must not use ':' or ',' on expected or command value"
    );

    $np->getopts();

}

function sendPassiveResult($status, $output)
{
    global $np;

    Cmd::serviceSendPassiveCheckResult(array(
        'time' => time(),
        'host_name' => $np->opts['hostname'],
        'service_description' => $np->opts['service-description'],
        'check_result' => $status,
        'plugin_output' => $output
    ));
}

function getConnector()
{
    global $np;

    $connector = isTelnet() ? new TelnetConnector : new SSHConnector;
    $credential = new UserCredentials();
    $credential->retrieve($np->opts['credential-id']);

    if (isset($np->opts['verbose'])) {
        $connector->set_verbose();
    }

    $config = new StdClass();
    $config->host = $np->opts['host_address'];
    $config->port = $np->opts['port'];
    $config->username = $credential->get_username();
    $config->password = $credential->get_password();

    try {
        $connector->open($config);
    } catch (ConnectorException $e) {
        quit($e->getMessage(), CRITICAL);
    }

    return $connector;
}

/*******************************************************************************
* Executa os comandos necessários no equipamento para obter a configuração
*
* @return String $configuration Configurações obtidas do ssh.
********************************************************************************/
function getConfiguration($connector)
{
    global $np;

    try {
        $configuration = executeCommands($connector);
    } catch (ConnectorException $e) {
        quit($e->getMessage(), CRITICAL);
    } catch (InvalidArgumentException $e) {
        quit($e->getMessage());
    }

    $connector->close();

    return removeTrashFromConfiguration($configuration);
}

function isTelnet()
{
    global $np;
    return ('telnet' == $np->opts['protocol']);
}

function removeTrashFromConfiguration($configuration)
{
    global $np;

    removeCommand($np->opts['backup-command'], $configuration);

    foreach ($np->opts['commands'] as $command) {
        if (isset($command[1])) {
            removeCommand($command[1], $configuration);
        }
    }

    $configuration = preg_replace("/^(\r|\n)+/m", "", $configuration);
    # Remove datetime sintaxe: Mon Nov 18 15:33:57 2019
    $configuration = preg_replace(
        '/([a-zA-Z]{3})\s([a-zA-Z]{3})\s([0-9]{1,2})\s([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})\s([0-9]{4})/m',
        '',
        $configuration
    );
    return trim(preg_replace(
        '/' . $np->opts['hostname'] . '(\s+)?#/',
        '',
        $configuration
    ));
}

function removeCommand($command, &$configuration)
{
    $configuration = preg_replace(
        '/.*' . str_replace("/", "\/", $command). '/',
        '',
        $configuration
    );
}

function quit($msg, $status = UNKNOWN)
{
    global $np;
    sendPassiveResult($status, $msg);
    $np->nagios_die($msg);
}

/*******************************************************************************
* Envia as informações passadas por parametro para o servidor OpSCM no OpMon
********************************************************************************/

function store($data)
{
    global $np;

    if (empty($data)) {
        $np->nagios_die("Invalid file content");
    }

    $scm = new Scm();
    $out = new StdClass();

    $scm->set_host_id($np->opts['host_id']);
    $scm->set_name('switch.cfg');
    $scm->set_content(base64_encode($data));
    $scm->set_md5_content(md5($data));

    $out->status = OK;
    $out->output = "Configuration file has no changes since last update";

    //Was file changed since the last commit?
    if ($scm->store() !== false) {
        $out->status = CRITICAL;
        $out->output = "The configuration file has changes";
    }

    unset($scm);

    return $out;
}

function getHostInformation()
{
    global $np;

    $host = get_host_by_name($np->opts['hostname']);
    if (! $host instanceof Host) {
        $np->nagios_exit(CRITICAL, "Host not found");
    }

    $np->opts['host_address'] = $host->get_address();
    $np->opts['host_id'] = $host->get_host_id();
}

function processCommands($commands)
{
    if (! empty($commands)) {
        $commands = explode(',', $commands);
        return array_map(
            'explode',
            array_fill(0, count($commands), ':'),
            $commands
        );
    }
}

function forkMe()
{
    global $np;

    $command = sprintf(
        "%s -P %s -H %s -i %d -m %s -p %d -c '%s' -s %s %s --run > /dev/null 2>/dev/null &",
        realpath(__FILE__),
        $np->opts['protocol'],
        $np->opts['hostname'],
        $np->opts['credential-id'],
        $np->opts['model'],
        $np->opts['port'],
        $np->opts['backup-command'],
        $np->opts['service-description'],
        (! empty($np->opts['command-list'])) ? "-l '{$np->opts['command-list']}'" : ""
    );

    if (isset($np->opts['verbose'])) {
        echo "Backup Command Line: $command\n";
    }

    shell_exec($command);
}

function executeCommands(&$connector)
{
    global $np;

    if (! empty($np->opts['commands'])) {
        $connector->set_timeout($np->opts['timeout']);
        $connector->executeBatch($np->opts['commands']);
        return $connector->get_buffer();
    }

    if ('3com' == $np->opts['model']) {
        if (isTelnet()) {
            $connector->set_timeout(10)
                ->execute('/login/i', '')
                ->execute('/>/', "\n");

            $connector->set_timeout(500)->clear_buffer();
        }

        $connector->execute('/>/', $np->opts['backup-command'])
               ->execute('/return/i', 'quit');

        return $connector->get_buffer();
    }

    if ('dlink' == $np->opts['model']) {
        $connector->set_timeout(500);

        $connector->execute('/:4#/', $np->opts['backup-command'])
               ->clear_buffer()
               ->execute('/\[7ma/', 'a')
               ->execute('/End of configuration/', 'logout');

        return $connector->get_buffer();
    }

    if ('cisco' == $np->opts['model']) {
        $connector->execute('/#/', $np->opts['backup-command'])
               ->execute('/^end$/', '') //End of configuration
               ->execute('/#/', 'logout');

        return $connector->get_buffer();
    }

    if ('extreme' == $np->opts['model']) {
        $connector->execute('/#/', "")->clear_buffer();

        $connector->execute('/#/', $np->opts['backup-command'] . "\n")
            ->execute('/\.\d #/', "logout\n");

        return $connector->get_buffer();

    }

    quit("Model not implemented yet");
}

function filterParameter()
{
    global $np;

    $commands = array(
        '3com' => 'display current-configuration configuration',
        'cisco' => 'show running-config',
        'dlink' => 'show config current_config',
        'extreme' => "show configuration",
    );

    if (! isset($np->opts['hostname'])) {
        $np->nagios_die("Hostname is required");
    }

    if (! isset($np->opts['credential-id'])) {
        $np->nagios_die("Credential ID is required");
    }

    if (! preg_match('/^(cisco|3com|d-?link|extreme|other)$/i', $np->opts['model'])) {
        $np->nagios_die("Switch model is not support. Use cisco, 3com, dlink or extreme");
    }

    if (! preg_match('/^(telnet|ssh)$/', $np->opts['protocol'])) {
        $np->nagios_die("Protocol is not support. Use ssh or telnet");
    }

    $np->opts['model'] = strtolower($np->opts['model']);
    $np->opts['model'] = str_replace('-', '', $np->opts['model']);

    $np->opts['backup-command'] = (is_string($np->opts['backup-command']))
        ? $np->opts['backup-command']
        : $commands[ $np->opts['model'] ];

    $np->opts['service-description']
        = $np->opts['service-description'] ?: 'Configuration_Backup';

    $protocol_port = isTelnet() ? 23 : 22;

    $np->opts['port'] = $np->opts['port'] ?: $protocol_port;

    $np->opts['commands'] = processCommands($np->opts['command-list']);

    $np->opts['timeout'] = $np->opts['timeout'] ?: 10;
}


/*******************************************************************************
* Rotina principal do programa
********************************************************************************/

function main()
{
    setNagios();

    global $np;

    if (isset($np->opts['man'])) {
        getGuideLine();
    }

    filterParameter();
    getHostInformation();

    $connector = getConnector();

    if (! isset($np->opts['run'])) {
        forkMe();
        $connector->close();
        $np->nagios_exit(OK, 'Starting backup...');
    }

    $configuration = getConfiguration($connector);
    $response = store($configuration);
    sendPassiveResult($response->status, $response->output);
    exit(OK);
}

main();
# vim: set smartindent tabstop=4 shiftwidth=4 softtabstop=4 expandtab
