#!/usr/bin/python

# desenvolvido por Wagner Garcez em 30/11/2018
# 
# + Ajustado monitoramento dos status das VMs, criei uma funcao para validar se o registro obtido refere-se de fato a VM requisitada. [Wagner Garcez, data: 19/02/2019] 
#
#

######################################################
# import das bibliotecas
######################################################
try:
        import signal
        import re
        import sys
        import time
        import os
        import XenAPI
        from datetime import datetime
        import time
        from optparse import OptionParser, OptionGroup
except Exception as e:
        print "UNKNOWN - %s" % e 
        sys.exit(3)

######################################################
# parse dos parametros recebidos na execucao do script
######################################################

def parse_args():
        parser = OptionParser()
        parser.add_option("-I", "--ipaddress", dest = "ipad", default = None, help = " IP of xen server")
        #parser.add_option("-P", "--poolname",  dest = "pool", default = None, help = " Name of pool")
        parser.add_option("-u", "--username",  dest = "user", default = None, help = " Username")
        parser.add_option("-p", "--password",  dest = "pa2s", default = None, help = " Password")
        parser.add_option("-o", "--option",    dest = "opti", default = None, help = " Options {storage_resource, vm_status,  host_state, licence}")
        parser.add_option("-w", "--warning",   dest = "warn", default = None, help = " Threshold of Warning")
        parser.add_option("-c", "--critical",  dest = "crit", default = None, help = " Threshold of Critical")
        parser.add_option("-s", "--storager",  dest = "stre", default = None, help = " Name of Storage Resource")
        parser.add_option("-m", "--vmachine",  dest = "vima", default = None, help = " Name of Virtual Machinee")
        parser.add_option("-v", "--verbose",   dest = "verb", default = False,help = " Verbose Mode",action="store_true")
        (options, args) = parser.parse_args()
        if len(sys.argv) < 2: usage()
        test = validate_args(options)
        if test != '':
                print test 
                sys.exit(3)
        return options

##########################################################
# validacao dos parametros recebidos na execucao do script
##########################################################

def validate_args(options):
        miss = ''
        if (options.ipad is None or options.user is None or options.pa2s is None or options.opti is None): # or options.host is None):
                miss = "UNKNOWN - Missing argument"
                if options.ipad is None:
                        miss += " -I <ip>"
                if options.user is None:
                        miss += " -u <user>"
                if options.pa2s is None:
                        miss += " -p <pass>"
                if options.opti is None:
                        miss += " -o <option>"
                return miss
        if options.stre is None and ( options.opti == 'pool_storage_resource' or options.opti == 'host_storage_resource'):
                miss += "UNKNOWN - Missing argument -s <pool_storage_resource|host_storage_resource>"
                return miss
        if options.vima is None and options.opti == 'vm_status':
                miss += "UNKNOWN - Missing argument -m <virtualmachine>"
                return miss
        if (options.warn is None or options.crit is None) and (options.opti == 'pool_storage_resource' or options.opti == 'host_storage_resource' or options.opti == 'licence') :
                miss += "UNKNOWN - Missing argument -w <warning> or -c <critical"
                return miss
        return miss

#######
# Help
#######

def usage():
        usage = '''
        Usage:
                script.py -I [ipaddress] -u [user] -p [pass] -o [option] -s [storageresource] -m [virtualmachine] -w [warning] -c [critical] -h [help] -v [verbose]

                -I  ... IP address 
                -u  ... User name to connect XenAPI
                -p  ... Password to connecet XenAPI
                -w  ... warning thresholds for Volume and Lic option
                -c  ... critical thresholds for Volume and Lic option 
                -s  ... Name of host storage resource to check
                -m  ... Name of virtual machine to check
                -o  ... Options of check (default: none)
                        : pool_storage_resource
                        : host_storage_resource
                        : vm_status
                        : host_state
                        : licence

                Sintax Command:
                        ---------------
                        Host_State:
                        ./check_xenserver.py -I <ipaddress> -u <user> -p <pass> -o host_state

                        Licence:
                        ./check_xenserver.py -I <ipaddress> -u <user> -p <pass> -o licence w <wvalue> -c <cvalue>

                        Vm_Status:
                        ./check_xenserver.py -I <ipaddress> -u <user> -p <pass> -o vm_status -m <vmname>
            
                        Host_Storage_Resource:
                        ./check_xenserver.py -I <ipaddress> -u <user> -p <pass> -o host_storage_resource -s <storage-resource-name> -w <wvalue> -c <cvalue>
            
                        Pool_Storage_Resource:
                        ./check_xenserver.py -I <ipaddress> -u <user> -p <pass> -o pool_storage_resource -s <storage_resource_name> -w <wvalue> -c <cvalue>
                        
            '''

        print usage
        sys.exit(3)


####################
# funcoes auxiliares
####################

def sizeof_fmt(num):
        try:
                return '%.2f' % (num/(1024.0**3))
        except Exception as e:
                quit_unknown(session, e, '[sizeof_fmt]')

def bytes_2_human_readable(number_of_bytes):
        if number_of_bytes < 0:
                raise ValueError("!!! number_of_bytes can't be smaller than 0 !!!")
        step_to_greater_unit = 1024.
        number_of_bytes = float(number_of_bytes)
        unit = 'bytes'
        if (number_of_bytes / step_to_greater_unit) >= 1:
                number_of_bytes /= step_to_greater_unit
                unit = 'KB'
        if (number_of_bytes / step_to_greater_unit) >= 1:
                number_of_bytes /= step_to_greater_unit
                unit = 'MB'
        if (number_of_bytes / step_to_greater_unit) >= 1:
                number_of_bytes /= step_to_greater_unit
                unit = 'GB'
        if (number_of_bytes / step_to_greater_unit) >= 1:
                number_of_bytes /= step_to_greater_unit
                unit = 'TB'
        precision = 1
        number_of_bytes = round(number_of_bytes, precision)
        return str(number_of_bytes) + ' ' + unit

def quit_unknown (session, e, f):
        session.xenapi.session.logout()
        print "UNKNOWN - %s %s" %(e,f)
        sys.exit(3)


def sigint_handler(signum, frame):
        print 'Stop pressing the CTRL+C!'
        sys.exit(3)

def valida_is_not_template(s, vms):
        for v in vms:
                r = s.xenapi.VM.get_record(v)
                if r['is_a_template'] is False:
                        return r
        print 'UNKNOWN - Something went wrong. Records found refers to a regular VM not ready for starting.'
        exit(3)

#def check_pool_name(session, pool_name):
#       if pool_name == options.pool:
#               return True
#       else:
#               if options.pool is None:
#                       quit_unknown(session, 'Favor informar o nome do pool','[check_pool_name]')
#               else:
#                       quit_unknown(session, 'Host not in pool ' + str(options.pool) , '[check_pool_name]')

#def check_sr_name(sr):
#       if sr == options.stre:
#               return True
#       else:
#               quit_unknown(session, 'SR %s not exist' + str(options.stre) , '[check_sr_name]')


#############################
# funcao de conexao com a API 
#############################

def xapi_connect(h, u, p):
        session = XenAPI.Session('https://'+h)
        try:
                session.xenapi.login_with_password(u, p)
                vprint ("\nMaster: " + str(h))
        except XenAPI.Failure, error: 
                msg = str(error.details[0])
                if "HOST_IS_SLAVE" in msg:
                        master = str(error.details[1] )
                        vprint ("\nMaster: " + str(master))
                        try: 
                                session = XenAPI.Session('https://'+master) 
                                session.login_with_password(u,p) 
                        except Exception, error: 
                                print "UNKNOWN - XenAPI Connection Error: " + error.details[2] + " with user " + error.details[1]
                                sys.exit(3)
                elif 'SESSION_AUTHENTICATION_FAILED' in msg:
                        print "UNKNOWN - XenAPI Connection Error: " + error.details[2] + " with user " + error.details[1]
                        sys.exit(3)
                else:
                        print "UNKNOWN - XenAPI Error : " + msg
                        sys.exit(3)
        except Exception, e :
                print "UNKNOWN - Connection Error: " + str(e)
                sys.exit(3)
        return session


####################################
# Validacao dos status das consultas
####################################

def validate_storage_resource(val, wa, cr, out):
        W = (float(val['size']) * float(wa) / 100)
        C = (float(val['size']) * float(cr) / 100)
        perf = " | 'used'=%s;%s;%s;0;%s 'free'=%s;%s;%s;0;%s 'total'=%s;%s;%s;0;%s" % ( val['used'], W, C, val['size'], val['free'], W, C, val['size'],val['size'], W, C, val['size'])
        if float(val['used']) < float(W):
                return 'OK - ' + str(out) + perf, '0'
        elif float(val['used']) < float(C):
                return 'WARNING - ' + str(out) + perf, '1'
        else:
                return 'CRITICAL - ' + str(out) + perf, '2'

def validate_status_hosts (val, wa, cr, out):
        if "Alive" in out:
                if val['enabled'] == True:
                        return 'OK - ' + str(out), '0' # + " | 'status'=2;0;0;;", '0'
                oconfig = val['other_config']
                if oconfig['MAINTENANCE_MODE'] == 'true':
                        out = 'WARNING - Is in MAINTENANCE_MODE, possible IT operation'
                        return out, '1' # + " | 'status'=1;0;0;;", '1'
                return 'CRITICAL - ' + str(out) + " | 'status'=0;0;0;;", '2'
        return 'CRITICAL - ' + str(out), '2' #+ " | 'status'=0;0;0;;", '2'

def validate_vms (val, wa, cr, out):
        out = out.encode('ascii', 'ignore').decode('ascii')
        if val == "Running":
                return 'OK - ' + str(out), '0' #+ " | 'status'=2;0;0;;", '0'
        else:
                return 'CRITICAL - ' + str(out), '2' #+ " | 'status'=0;0;0;;", '2'

def validate_licence (val, wa, cr, out):
        perf = " | 'days'="+ str(val)+";"+str(wa)+";"+str(cr)+";;"
        if int(val) > int(wa):
                return 'OK - ' + str(out) + perf, '0'
        elif int(val) > int(cr):
                return 'WARNING - ' + str(out) + perf, '1'
        elif int(val) <= int(cr):
                return 'CRITICAL - ' + str(out) + perf, '2'
        else:
                return 'UNKNOWN - ' + str(out) + perf, '3'

################################
# sub funcoes de consulta na API 
################################
def get_ref_pool(s):
        try:
                ref_pool = s.xenapi.pool.get_all()[0]
        except Exception as e:
                print "UNKNOWN - %s [get_ref_pool]" %e
                exit(3)
        return ref_pool

def get_ref_host(s):
        try:
                ref_host = s.xenapi.host.get_all()
        except Exception as e:
                print "UNKNOWN - %s [get_host_ref]" %e
                exit(3)
        return ref_host

def get_host_target(s, host_ref):
        for r in host_ref:
                try:
                        ip_host = s.xenapi.host.get_address(r)
                except Exception as e:
                        print "UNKNOWN - %s [get_host_ref]" %e
                        exit(3)
                if ip_host == options.ipad:
                        return ip_host, r
        print 'UNKNOWN - Something is wrong ...'
        exit(3) # fazer logout

def get_host_name(s, ref):
        try:
                host_name = s.xenapi.host.get_name_label(ref)
        except Exception as e:
                print "UNKNOWN - %s [get_host_name]" %e
                exit(3)
        return host_name

def get_pool_name(s, ref):
        try:
                host_name = s.xenapi.pool.get_name_label(ref)
        except Exception as e:
                print "UNKNOWN - %s [get_pool_name]" %e
                exit(3)
        return host_name

#########################################
# Consultas na API e parse das informacoes
#########################################

def check_host_storage_resource(session):
        vprint ('\nchecking host SR ...\n')
        ref_pool = get_ref_pool (session) 
        pool_name = get_pool_name(session, ref_pool) 
        if not pool_name:
                sr_ref = session.xenapi.SR.get_by_name_label(options.stre)
                if sr_ref: 
                        host_ref = get_ref_host(session) 
                        host_name = get_host_name(session, host_ref[0])
                        if host_ref:
                                record_sr = session.xenapi.SR.get_record(sr_ref[0])
                                data = {'size':0, 'free':0, 'used':0}
                                physical_size = int(record_sr['physical_size'])
                                physical_utilisation = int(record_sr['physical_utilisation'])
                                data['size'] = sizeof_fmt(physical_size)
                                data['used'] = sizeof_fmt(physical_utilisation)
                                data['free'] = sizeof_fmt((physical_size - physical_utilisation))
                                vprint('Disk Name: \'%s\'\n' % record_sr['name_label'])
                                vprint('Used\t: %s' % bytes_2_human_readable(physical_utilisation))
                                vprint('Free\t: %s' % bytes_2_human_readable(physical_size - physical_utilisation))
                                vprint('Size\t: %s' % bytes_2_human_readable(physical_size)+'\n')
                                disk_usage_perc = 100 * physical_utilisation/physical_size
                                output = 'Disk \'%s\' on Host \'%s\': %s used (%s%%)' %(options.stre, host_name, bytes_2_human_readable(physical_utilisation), disk_usage_perc)
                                return validate_storage_resource (data, int(options.warn), int(options.crit), output)
                        else:
                                out = 'UNKNOWN - No data found for host %s' % host_name
                                return out, '3'
                else:
                        out = 'UNKNOWN - Storage Resource %s not found' % options.stre
                        return out, '3'
        else:
                ip = ''
                host_ref = get_ref_host(session)
                ip, ref_host = get_host_target(session, host_ref)
                host_name = get_host_name(session, ref_host) 
                try:
                        sr_ref = session.xenapi.SR.get_by_name_label(options.stre)
                except Exception as e:
                        out = 'UNKNOWN - Could not get SR references: %s' % e
                        return out, '3'
                if sr_ref:
                        for s in sr_ref:
                                try:
                                        record_sr = session.xenapi.SR.get_record(s)
                                        pbd = record_sr['PBDs'][0]
                                        host_ref_by_pbd = session.xenapi.PBD.get_host(pbd)
                                except Exception as e:
                                        out = 'UNKNOWN - Could not get info of SR %s. %s' % (options.stre,e)
                                        return out, '3'
                                if ref_host == host_ref_by_pbd:
                                        ref_sr = s
                        try:
                                record_sr = session.xenapi.SR.get_record(ref_sr)
                        except Exception as e:
                                out = 'UNKNOWN - Could not get SR records. %s' % e
                                return out, '3'
                        if options.stre in record_sr['name_label']:
                                data = {'size':0, 'free':0, 'used':0}
                                physical_size = int(record_sr['physical_size'])
                                physical_utilisation = int(record_sr['physical_utilisation'])
                                data['size'] = sizeof_fmt(physical_size)
                                data['used'] = sizeof_fmt(physical_utilisation)
                                data['free'] = sizeof_fmt((physical_size - physical_utilisation))
                                vprint('Disk Name: \'%s\'\n' % record_sr['name_label'])
                                vprint('Used\t: %s' % bytes_2_human_readable(physical_utilisation))
                                vprint('Free\t: %s' % bytes_2_human_readable(physical_size - physical_utilisation))
                                vprint('Size\t: %s' % bytes_2_human_readable(physical_size)+'\n')
                                disk_usage_perc = 100 * physical_utilisation/physical_size
                                output = 'Disk \'%s\' on Host \'%s\': %s used (%s%%)' %(options.stre, host_name, bytes_2_human_readable(physical_utilisation), disk_usage_perc)
                                return validate_storage_resource (data, int(options.warn), int(options.crit), output)
                        else:
                                out = 'UNKNOWN - Storage Resource %s not found' % options.stre
                                return out, '3'
                else:
                        out = 'UNKNOWN - Could not get Storage Resource reference'
                        return out, '3'

def check_pool_storage_resource(session):
        vprint ('\nchecking pool SR ...\n')
        ref_pool = get_ref_pool(session)
        pool_name = get_pool_name(session, ref_pool) 
        if pool_name:
                pool_records = session.xenapi.pool.get_all_records()
                record_sr = session.xenapi.SR.get_record(pool_records[ref_pool]['default_SR'])
                if options.stre in record_sr['name_label']:
                        data = {'size':0, 'free':0, 'used':0}
                        physical_size = int(record_sr['physical_size'])
                        physical_utilisation = int(record_sr['physical_utilisation'])
                        data['size'] = sizeof_fmt(physical_size)
                        data['used'] = sizeof_fmt(physical_utilisation)
                        data['free'] = sizeof_fmt((physical_size - physical_utilisation))
                        vprint('Disk Name: \'%s\'\n' % record_sr['name_label'])
                        vprint('Used\t: %s' % bytes_2_human_readable(physical_utilisation))
                        vprint('Free\t: %s' % bytes_2_human_readable(physical_size - physical_utilisation))
                        vprint('Size\t: %s' % bytes_2_human_readable(physical_size)+'\n')
                        disk_usage_perc = 100 * physical_utilisation/physical_size
                        output = 'Disk \'%s\' on Pool \'%s\': %s used (%s%%)' %(options.stre, pool_name, bytes_2_human_readable(physical_utilisation), disk_usage_perc)
                        return validate_storage_resource (data, int(options.warn), int(options.crit), output)
                else:
                        out = 'UNKNOWN - SR \'%s\' not exist' %options.stre
                        return out, '3'
        else: 
                out = 'UNKNOWN - Host is standalone'
                return out, '3'

def check_lic (session, warn, crit):
        host_ref = get_ref_host(session)
        ip, ref_host = get_host_target(session, host_ref)
        host_name = get_host_name(session, ref_host) 
        lic = session.xenapi.host.get_license_params(ref_host)
        iso8601 = "%Y%m%dT%H:%M:%SZ"
        now = time.strftime(iso8601, time.gmtime(time.time()))
        date_now = datetime.strptime(now,iso8601)
        date_expiry = datetime.strptime(lic['expiry'],iso8601)
        delta = date_expiry - date_now
        vprint("\nHostname\t: " + host_name)
        vprint("Date Now\t: " + str(date_now.strftime("%d/%m/%Y")))
        vprint("Date Expiry\t: " + str(date_expiry.strftime("%d/%m/%Y")))
        vprint("Days of expiry\t: " + str(delta.days)+"\n")
        output = "Licence on Host " + host_name + " expire in " + str(delta.days) + " days. Date: " + str(date_expiry.strftime("%d/%m/%Y"))+""
        return validate_licence (delta.days, warn, crit, output)


def check_vms(session, vm_name):
        vm = session.xenapi.VM.get_by_name_label(vm_name)
        ref_pool = get_ref_pool(session) 
        pool_name = get_pool_name (session, ref_pool) 
        if vm:
                if len(vm) == 1: 
                                        record = session.xenapi.VM.get_record(vm[0])
                else:
                                        record = valida_is_not_template(session, vm)
                vprint('\nVm Name\t: ' + record['name_label'])
                vprint('Status\t: ' + record['power_state'])
                if pool_name: vprint("Poolname: " + pool_name + '\n')
                output = "VM '%s' is %s" %(record['name_label'], record['power_state'])
                return validate_vms (record['power_state'], '', '', output)
        else:
                out = "UNKNOWN - Please check Virtual Machine Name: '%s'" % vm_name
                return out, 3

def check_host(session):
        host_ref = get_ref_host(session) 
        ip, ref_host = get_host_target(session, host_ref)
        host_name = get_host_name(session, ref_host) 
        vprint ('\nchecking host state ...\n')
        vprint ('Host Name : ' + str(host_name)) 
        try:
                status = session.xenapi.host_metrics.get_live(session.xenapi.host.get_metrics(ref_host))
                hrecord = session.xenapi.host.get_record(ref_host)
        except Exception as e:
                out = 'UNKNOWN - Could not get host records. %s' % e
                return out, '3'
        if status == True:
                vprint('Host State: ' + str(status)+'\n')
                output = host_name + " is Alive."
                return validate_status_hosts (hrecord, None, None, output)
        else:
                output = host_name + " is Dead."
                return validate_status_hosts (hrecord, None, None, output)

##
## Debug
##

def q():
        exit(3)

def info(session):
        href = get_ref_host(session)
        #print '\n Total hosts in pool: ' + str(len(href))
        try:
                pool = str(session.xenapi.pool.get_name_label(session.xenapi.pool.get_all()[0]))
        except Exception as e:
                out = 'UNKNOWN - Could not get host name'
                return out, '3'
        if pool:
                print "\n Hosts in pool '%s' " % (pool) #and List of VMs currently resident on host'" % (pool))
                for h in href:
                        print ('\n  -> ' + str(session.xenapi.host.get_name_label(h)) +' ('+ str(session.xenapi.host.get_address(h))+')')
                        version_Xen = session.xenapi.host.get_software_version(h)
                        vref = session.xenapi.host.get_resident_VMs(h)
                        for v in vref:
                                vname = session.xenapi.VM.get_name_label(v).encode('ascii', 'ignore').decode('ascii')
                                regexp = '.*%s*' % (get_host_name(session,h))
                                if not re.search(regexp, vname):
                                        record = session.xenapi.VM.get_record(v)
                                        print '\t- ' + vname + '\t' #+ record['power_state']
                        print '\n\t- VMs running in host:  ' + str( len(vref) - 1)
                        print('\n')
                        print "  "+str(version_Xen['product_brand'])+" Version: "+ str(version_Xen['product_version']), 0
        else:
                print '\n Host standalone '
                for h in href:
                        print ('\n  -> ' + str(session.xenapi.host.get_name_label(h)) +' ('+ str(session.xenapi.host.get_address(h))+')')
                        version_Xen = session.xenapi.host.get_software_version(h)
                        vref = session.xenapi.host.get_resident_VMs(h)
                        for v in vref:
                                vname = session.xenapi.VM.get_name_label(v).encode('ascii', 'ignore').decode('ascii')
                                regexp = '.*%s*' % (get_host_name(session,h))
                                if not re.search(regexp, vname):
                                        record = session.xenapi.VM.get_record(v)
                                        print '\t- ' + vname + '\t' #+ record['power_state']
                        print '\n\t- VMs running in host:  ' + str( len(vref) - 1)
                        print('\n')
                        print "  "+str(version_Xen['product_brand'])+" Version: "+ str(version_Xen['product_version']), 0
                        out = 'Not to show'
                        return out, '3'

def vprint(obj):
        if options.verb: print(obj)
        return True

#############
# funcao main
#############

def main():
        out = ''
        code = 3
        session = xapi_connect(options.ipad, options.user, options.pa2s)
        opt = options.opti
        if opt == 'host_storage_resource':
                out, code = check_host_storage_resource(session) 
        elif opt == 'pool_storage_resource':
                out, code = check_pool_storage_resource(session) 
        elif opt == 'host_state':
                out, code = check_host(session)
        elif opt == 'vm_status':
                out, code = check_vms(session, options.vima)
        elif opt == 'licence':
                out, code = check_lic(session, options.warn, options.crit)
        elif opt == 'info':
                info(session)
        else:
                (out , code ) = "UNKNOWN - Invalid option: '-o " + str(opt) + "' " , '3'
        session.xenapi.session.logout()
        print out
        sys.exit(int(code))

########################
# chamada da funcao main
########################

if __name__ == "__main__":
        options = parse_args()
        signal.signal(signal.SIGINT, sigint_handler)

main()
