#! /usr/bin/python

#####################################################
# Script para monitorar a porcentagem de ciclos		#
#  de escrita restante para os SSDs.				#
#													#
#       Criado por Rodrigo Ervella					#
#####################################################

import re
import commands
from sys import exit
from optparse import OptionParser

parser = OptionParser()
parser.add_option("-i", "--ip", dest="ip", help="iDrac IP", default=None)
parser.add_option("-C", "--community", dest="com", help="SNMP community", default=None)
parser.add_option("-d", "--disk", dest="disk", help="Disk name", default=None)
parser.add_option("-w", "--warning", dest="w", type=int, help="Warning threshold", default=None)
parser.add_option("-c", "--critical", dest="c", type=int, help="Critical threshold", default=None)
(opts,args) = parser.parse_args()

if opts.ip == None:
  parser.error("Need iDrac IP (-i)")
if opts.com == None:
  parser.error("Need SNMP community (-C)")
if opts.disk == None:
  parser.error("Need disk name (-d)")
if opts.w == None:
  parser.error("Need warning threshold (-w)")
if opts.c == None:
  parser.error("Need critical threshold (-c)")

cmdName=("snmpwalk -v 2c -c "+opts.com+" "+opts.ip+" 1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.2")
#gera uma lista com o retorno da consulta SNMP
names=((commands.getoutput(cmdName)).split('\n'))

if "Timeout: No Response" in names[0]:
  print("Timeout: no response from " + opts.ip + ". Check if IP and community are correct or if SNMPD is up.")
  exit(3)
else:
  #variavel para controlar se o disco informado existe. 0 = nao existe / 1 = existe
  e = 0
  for n in names:
	#valida se o nome do disco existe
    if (re.findall(r'STRING: "(.*)"',n))[0] ==  opts.disk:
      e = 1
	  #armazena o indice do disco na OID
      i = (re.findall(r'2.(\d+) = STRING.*"', n))[0]
      #coleta o valor do cycle de acordo com o indice do disco
      diskCycle=(commands.getoutput("snmpwalk -v 2c -Ovq -c "+opts.com+" "+opts.ip+" 1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.49."+i))

  if e == 1:
    perf=("diskCycle="+diskCycle+";"+str(opts.w)+";"+str(opts.c)+";;")
    if int(diskCycle) <= opts.c:
      print("CRITICAL - Disk cycle is "+diskCycle+"%. | "+perf)
      exit(2)
    elif (int(diskCycle) > opts.c) and (int(diskCycle) <= opts.w):
      print("WARNING - Disk cycle is "+diskCycle+"%. | "+perf)
      exit(1)
    else:
      print("OK - Disk cycle is "+diskCycle+"%. | "+perf)
      exit(0) 
  else:
    print("UNKNOWN - Disk not found.")
    exit(3)
